<!DOCTYPE html>
<html lang="{{ locale.code }}">
<head>
  <meta charset="utf-8">
  <meta http-equiv="Content-Type" content="text/html;charset=UTF-8" />
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  {% block head %}{% end %}
  {% block css %}
    {% for path in handler.css() %}
      <link rel="stylesheet" href="{{ static_url(path) }}" type="text/css" />
    {% end %}
  {% end %}
</head>
<body class="wrapper">
  {% block top %}
    <header>
    Hello world
    </header>
  {% end %}
  <div class="wrapper-body">
    {% block body %}{% end %}
  </div>

  {% block bottom %}
  <footer>
    2015
  </footer>
  {% end %}
  
  {% block js %}
    {% for path in handler.js() %}
      <script src="{{ static_url(path) }}" type="text/javascript"></script>
    {% end %}
  {% end %}
  {% block inline %}{% end %}
</body>
</html>
