/*
Conventions:

- Use camelClass name
- Use dash to indicate child element
- Use .xFunction prefix if that class is used by Javascript
- Use simple selectors

References:

  http://engineering.appfolio.com/2012/11/16/css-architecture/
  http://philipwalton.github.io/solved-by-flexbox/
*/

/* Reset */
* { margin: 0; padding: 0; }
body,div,span,li,button,input,select,textarea,dt,dl,a,p,th,td {
  font-family: Arial,HiraKakuProN-W3,Meiryo,sans-serif;
  font-size: 1rem;
}
button,input,select,textarea,p,th,td {
  padding: 0.3rem;
}
input,select,textarea {
  border: solid 1px;
  border-color: darkgrey;
}
a {
  text-decoration: none;
  color: darkblue;
}
a:hover {
  text-decoration: underline;
}

/* Sticky footer */
.wrapper {
  display: flex;
  min-height: 100vh;
  flex-direction: column;
}
.wrapper-body {
  flex: 1;
}

